package analysisOperations;

import gui.Option;
import ij.measure.ResultsTable;
import operations.Operation;

public class FilterMeasurementsOperation extends Operation {
	protected ResultsTable measurements;
	protected ResultsTable resultMeasurements;
	
	protected Option minArea;
	protected Option maxArea;
	protected Option minMean;
	protected Option maxMean;
	protected Option minStdDev;
	protected Option maxStdDev;
	protected Option minMode;
	protected Option maxMode;
	protected Option minMin;
	protected Option maxMin;
	protected Option minMax;
	protected Option maxMax;
	protected Option minCentroidX;
	protected Option maxCentroidX;
	protected Option minCentroidY;
	protected Option maxCentroidY;
	protected Option minCenterOfMassX;
	protected Option maxCenterOfMassX;
	protected Option minCenterOfMassY;
	protected Option maxCenterOfMassY;
	protected Option minPerimeter;
	protected Option maxPerimeter;
	protected Option minBoundingBoxX;
	protected Option maxBoundingBoxX;
	protected Option minBoundingBoxY;
	protected Option maxBoundingBoxY;
	protected Option minBoundingBoxWidth;
	protected Option maxBoundingBoxWidth;
	protected Option minBoundingBoxHeight;
	protected Option maxBoundingBoxHeight;
	protected Option minMajor;
	protected Option maxMajor;
	protected Option minMinor;
	protected Option maxMinor;
	protected Option minAngle;
	protected Option maxAngle;
	protected Option minCircularity;
	protected Option maxCircularity;
	protected Option minFeret;
	protected Option maxFeret;
	protected Option minIntDensity;
	protected Option maxIntDensity;
	protected Option minMedian;
	protected Option maxMedian;
	protected Option minSkew;
	protected Option maxSkew;
	protected Option minKurt;
	protected Option maxKurt;

	/**
	 * 
	 */
	private static final long serialVersionUID = -5098569786394788296L;

	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		parameterTypes = new Class[1];
		parameterTypes[0] = Class.forName("ij.measure.ResultsTable");
		parameterNames = new String[1];
		parameterNames[0] = "Measurements";
		optionsNames = new String[48];
		optionsNames[0] = "min area";
		optionsNames[1] = "max area";
		optionsNames[2] = "min mean";
		optionsNames[3] = "max mean";
		optionsNames[4] = "min std. dev.";
		optionsNames[5] = "max std. dev.";
		optionsNames[6] = "min mode";
		optionsNames[7] = "max mode";
		optionsNames[8] = "min min intensity";
		optionsNames[9] = "max min intensity";
		optionsNames[10] = "min max intensity";
		optionsNames[11] = "max max intensity";
		optionsNames[12] = "min centroid x";
		optionsNames[13] = "max centroid x";
		optionsNames[14] = "min centroid y";
		optionsNames[15] = "max centroid y";
		optionsNames[16] = "min center of mass x";
		optionsNames[17] = "max center of mass x";
		optionsNames[18] = "min center of mass y";
		optionsNames[19] = "max center of mass y";
		optionsNames[20] = "min perimeter";
		optionsNames[21] = "max perimeter";
		optionsNames[22] = "min bounding box x";
		optionsNames[23] = "max bounding box x";
		optionsNames[24] = "min bounding box y";
		optionsNames[25] = "max bounding box y";
		optionsNames[26] = "min bounding box width";
		optionsNames[27] = "max bounding box width";
		optionsNames[28] = "min bounding box height";
		optionsNames[29] = "max bounding box height";
		optionsNames[30] = "min major";
		optionsNames[31] = "max major";
		optionsNames[32] = "min minor";
		optionsNames[33] = "max minor";
		optionsNames[34] = "min angle";
		optionsNames[35] = "max angle";
		optionsNames[36] = "min circularity";
		optionsNames[37] = "max circularity";
		optionsNames[38] = "min feret";
		optionsNames[39] = "max feret";
		optionsNames[40] = "min int. density";
		optionsNames[41] = "max int. density";
		optionsNames[42] = "min median";
		optionsNames[43] = "max median";
		optionsNames[44] = "min skew";
		optionsNames[45] = "max skew";
		optionsNames[46] = "min kurt";
		optionsNames[47] = "max kurt";
		resultTypes = new Class[1];
		resultTypes[0] = Class.forName("ij.measure.ResultsTable");
		resultNames = new String[1];
		resultNames[0] = "ResultMeasurements";
	}
	
	/**
	 * @return Returns the measurements.
	 */
	public ResultsTable getMeasurements() {
		return measurements;
	}
	/**
	 * @param measurements The measurements to set.
	 */
	public void setMeasurements(ResultsTable measurements) {
		this.measurements = measurements;
	}
	/**
	 * @return Returns the resultMeasurements.
	 */
	public ResultsTable getResultMeasurements() {
		return resultMeasurements;
	}
	/**
	 * @param resultMeasurements The resultMeasurements to set.
	 */
	public void setResultMeasurements(ResultsTable resultMeasurements) {
		this.resultMeasurements = resultMeasurements;
	}
	
	/**
	 * 
	 */
	protected void cleanUpInput() {
		this.setMeasurements(null);
	}
	
	/**
	 * 
	 */
	protected void showResult() {
		this.getResultMeasurements().show("objects");
	}
	
	/**
	 *  
	 */
	public void doIt() {
		resultMeasurements = new ResultsTable();
		resultMeasurements.reset();
		ResultsTable measurements = this.getMeasurements();
		for (int i=0; i<measurements.getCounter(); i++) {
			if (this.shouldAccept(i)){
				resultMeasurements.incrementCounter();
				for (int j=0; j<ResultsTable.MAX_COLUMNS; j++) {
					if (measurements.columnExists(j)) {
					resultMeasurements.addValue(j, measurements.getValue(j, i));
					}
				}
			}
		}
	}

	private boolean shouldAccept(int i) {
		ResultsTable measurements = this.getMeasurements();
		if (!minArea.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.AREA, i)<minArea.getFloatValue()) return false; 
		}
		if (!maxArea.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.AREA, i)>maxArea.getFloatValue()) return false; 
		}
		if (!minMean.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MEAN, i)<minMean.getFloatValue()) return false; 
		}
		if (!maxMean.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MEAN, i)>maxMean.getFloatValue()) return false; 
		}
		if (!minStdDev.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.STD_DEV, i)<minStdDev.getFloatValue()) return false; 
		}
		if (!maxStdDev.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.STD_DEV, i)>maxStdDev.getFloatValue()) return false; 
		}
		if (!minMode.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MODE, i)<minMode.getFloatValue()) return false; 
		}
		if (!maxMode.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MODE, i)>maxMode.getFloatValue()) return false; 
		}
		if (!minMin.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MIN, i)<minMin.getFloatValue()) return false; 
		}
		if (!maxMin.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MIN, i)>maxMin.getFloatValue()) return false; 
		}
		if (!minMax.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MAX, i)<minMax.getFloatValue()) return false; 
		}
		if (!maxMax.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MAX, i)>maxMax.getFloatValue()) return false; 
		}
		if (!minCentroidX.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.X_CENTROID, i)<minCentroidX.getFloatValue()) return false; 
		}
		if (!maxCentroidX.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.X_CENTROID, i)>maxCentroidX.getFloatValue()) return false; 
		}
		if (!minCentroidY.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.Y_CENTROID, i)<minCentroidY.getFloatValue()) return false; 
		}
		if (!maxCentroidY.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.Y_CENTROID, i)>maxCentroidY.getFloatValue()) return false; 
		}
		if (!minCenterOfMassX.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.X_CENTER_OF_MASS, i)<minCenterOfMassX.getFloatValue()) return false; 
		}
		if (!maxCenterOfMassX.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.X_CENTER_OF_MASS, i)>maxCenterOfMassX.getFloatValue()) return false; 
		}
		if (!minCenterOfMassY.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.Y_CENTER_OF_MASS, i)<minCenterOfMassY.getFloatValue()) return false; 
		}
		if (!maxCenterOfMassY.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.Y_CENTER_OF_MASS, i)>maxCenterOfMassY.getFloatValue()) return false; 
		}
		if (!minPerimeter.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.PERIMETER, i)<minPerimeter.getFloatValue()) return false; 
		}
		if (!maxPerimeter.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.PERIMETER, i)>maxPerimeter.getFloatValue()) return false; 
		}
		if (!minBoundingBoxX.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ROI_X, i)<minBoundingBoxX.getFloatValue()) return false; 
		}
		if (!maxBoundingBoxX.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ROI_X, i)>maxBoundingBoxX.getFloatValue()) return false; 
		}
		if (!minBoundingBoxY.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ROI_Y, i)<minBoundingBoxY.getFloatValue()) return false; 
		}
		if (!maxBoundingBoxY.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ROI_Y, i)>maxBoundingBoxY.getFloatValue()) return false; 
		}
		if (!minBoundingBoxWidth.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ROI_WIDTH, i)<minBoundingBoxWidth.getFloatValue()) return false; 
		}
		if (!maxBoundingBoxWidth.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ROI_WIDTH, i)>maxBoundingBoxWidth.getFloatValue()) return false; 
		}
		if (!minBoundingBoxHeight.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ROI_HEIGHT, i)<minBoundingBoxHeight.getFloatValue()) return false; 
		}
		if (!maxBoundingBoxHeight.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ROI_HEIGHT, i)>maxBoundingBoxHeight.getFloatValue()) return false; 
		}
		if (!minMajor.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MAJOR, i)<minMajor.getFloatValue()) return false; 
		}
		if (!maxMajor.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MAJOR, i)>maxMajor.getFloatValue()) return false; 
		}
		if (!minMinor.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MINOR, i)<minMinor.getFloatValue()) return false; 
		}
		if (!maxMinor.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MINOR, i)>maxMinor.getFloatValue()) return false; 
		}
		if (!minAngle.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ANGLE, i)<minAngle.getFloatValue()) return false; 
		}
		if (!maxAngle.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.ANGLE, i)>maxAngle.getFloatValue()) return false; 
		}
		if (!minCircularity.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.CIRCULARITY, i)<minCircularity.getFloatValue()) return false; 
		}
		if (!maxCircularity.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.CIRCULARITY, i)>maxCircularity.getFloatValue()) return false; 
		}
		if (!minFeret.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.FERET, i)<minFeret.getFloatValue()) return false; 
		}
		if (!maxFeret.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.FERET, i)>maxFeret.getFloatValue()) return false; 
		}
		if (!minIntDensity.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.INTEGRATED_DENSITY, i)<minIntDensity.getFloatValue()) return false; 
		}
		if (!maxIntDensity.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.INTEGRATED_DENSITY, i)>maxIntDensity.getFloatValue()) return false; 
		}
		if (!minMedian.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MEDIAN, i)<minMedian.getFloatValue()) return false; 
		}
		if (!maxMedian.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.MEDIAN, i)>maxMedian.getFloatValue()) return false; 
		}
		if (!minSkew.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.SKEWNESS, i)<minSkew.getFloatValue()) return false; 
		}
		if (!maxSkew.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.SKEWNESS, i)>maxSkew.getFloatValue()) return false; 
		}
		if (!minKurt.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.KURTOSIS, i)<minKurt.getFloatValue()) return false; 
		}
		if (!maxKurt.getValue().equals("null")) {
			if (measurements.getValue(ResultsTable.KURTOSIS, i)>maxKurt.getFloatValue()) return false; 
		}
		return true;
	}

	public void connectOptions() {
		this.minArea= (Option) this.options.getOptions().elementAt(0);
		this.maxArea = (Option) this.options.getOptions().elementAt(1);
		this.minMean = (Option) this.options.getOptions().elementAt(2);
		this.maxMean= (Option) this.options.getOptions().elementAt(3);
		this.minStdDev = (Option) this.options.getOptions().elementAt(4);
		this.maxStdDev = (Option) this.options.getOptions().elementAt(5);
		this.minMode= (Option) this.options.getOptions().elementAt(6);
		this.maxMode = (Option) this.options.getOptions().elementAt(7);
		this.minMin = (Option) this.options.getOptions().elementAt(8);
		this.maxMin= (Option) this.options.getOptions().elementAt(9);
		this.minMax = (Option) this.options.getOptions().elementAt(10);
		this.maxMax = (Option) this.options.getOptions().elementAt(11);
		this.minCentroidX= (Option) this.options.getOptions().elementAt(12);
		this.maxCentroidX = (Option) this.options.getOptions().elementAt(13);
		this.minCentroidY = (Option) this.options.getOptions().elementAt(14);
		this.maxCentroidY= (Option) this.options.getOptions().elementAt(15);
		this.minCenterOfMassX = (Option) this.options.getOptions().elementAt(16);
		this.maxCenterOfMassX = (Option) this.options.getOptions().elementAt(17);
		this.minCenterOfMassY = (Option) this.options.getOptions().elementAt(18);
		this.maxCenterOfMassY = (Option) this.options.getOptions().elementAt(19);
		this.minPerimeter = (Option) this.options.getOptions().elementAt(20);
		this.maxPerimeter = (Option) this.options.getOptions().elementAt(21);
		this.minBoundingBoxX = (Option) this.options.getOptions().elementAt(22);
		this.maxBoundingBoxX = (Option) this.options.getOptions().elementAt(23);
		this.minBoundingBoxY = (Option) this.options.getOptions().elementAt(24);
		this.maxBoundingBoxY = (Option) this.options.getOptions().elementAt(25);
		this.minBoundingBoxWidth = (Option) this.options.getOptions().elementAt(26);
		this.maxBoundingBoxWidth = (Option) this.options.getOptions().elementAt(27);
		this.minBoundingBoxHeight = (Option) this.options.getOptions().elementAt(28);
		this.maxBoundingBoxHeight = (Option) this.options.getOptions().elementAt(29);
		this.minMajor = (Option) this.options.getOptions().elementAt(30);
		this.maxMajor = (Option) this.options.getOptions().elementAt(31);
		this.minMinor = (Option) this.options.getOptions().elementAt(32);
		this.maxMinor= (Option) this.options.getOptions().elementAt(33);
		this.minAngle = (Option) this.options.getOptions().elementAt(34);
		this.maxAngle = (Option) this.options.getOptions().elementAt(35);
		this.minCircularity = (Option) this.options.getOptions().elementAt(36);
		this.maxCircularity = (Option) this.options.getOptions().elementAt(37);
		this.minFeret = (Option) this.options.getOptions().elementAt(38);
		this.maxFeret= (Option) this.options.getOptions().elementAt(39);
		this.minIntDensity = (Option) this.options.getOptions().elementAt(40);
		this.maxIntDensity = (Option) this.options.getOptions().elementAt(41);
		this.minMedian= (Option) this.options.getOptions().elementAt(42);
		this.maxMedian = (Option) this.options.getOptions().elementAt(43);
		this.minSkew = (Option) this.options.getOptions().elementAt(44);
		this.maxSkew= (Option) this.options.getOptions().elementAt(45);
		this.minKurt = (Option) this.options.getOptions().elementAt(46);
		this.maxKurt = (Option) this.options.getOptions().elementAt(47);
	}
}
