package cellCounter;

import gui.MultiChannelListEditor;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JFrame;
import java.awt.Toolkit;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JDesktopPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

public class CellCounterView extends JFrame implements Observer {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JScrollPane imageScrollPane = null;
	private JTable imageTable = null;
	private JDesktopPane imageDesktopPane = null;
	private JMenuBar jJMenuBar = null;
	private JMenu fileMenu = null;
	private JMenuItem openMenuItem = null;
	private CellCounter model;
	/**
	 * This is the default constructor
	 */
	public CellCounterView() {
		super();
		initialize();
	}

	public CellCounterView(CellCounter counter) {
		super();
		this.model = counter;
		counter.addObserver(this);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(555, 380);
		this.setJMenuBar(getJJMenuBar());
		this.setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("/gui/icon.gif")));
		this.setContentPane(getJContentPane());
		this.setTitle("MRI Cell Counter");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getImageScrollPane(), java.awt.BorderLayout.WEST);
			jContentPane.add(getImageDesktopPane(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes imageScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getImageScrollPane() {
		if (imageScrollPane == null) {
			imageScrollPane = new JScrollPane();
			imageScrollPane.setPreferredSize(new java.awt.Dimension(120,419));
			imageScrollPane.setViewportView(getImageTable());
		}
		return imageScrollPane;
	}

	/**
	 * This method initializes imageTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getImageTable() {
		if (imageTable == null) {
			imageTable = new JTable();
			imageTable.setSize(new java.awt.Dimension(100,80));
			imageTable.setPreferredSize(new java.awt.Dimension(90,80));
		}
		return imageTable;
	}

	/**
	 * This method initializes imageDesktopPane	
	 * 	
	 * @return javax.swing.JDesktopPane	
	 */
	private JDesktopPane getImageDesktopPane() {
		if (imageDesktopPane == null) {
			imageDesktopPane = new JDesktopPane();
		}
		return imageDesktopPane;
	}

	/**
	 * This method initializes jJMenuBar	
	 * 	
	 * @return javax.swing.JMenuBar	
	 */
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getJMenu());
		}
		return jJMenuBar;
	}

	/**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu();
			fileMenu.setText("File");
			fileMenu.add(getOpenMenuItem());
		}
		return fileMenu;
	}

	/**
	 * This method initializes openMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getOpenMenuItem() {
		if (openMenuItem == null) {
			openMenuItem = new JMenuItem();
			openMenuItem.setText("open...");
			openMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					MultiChannelListEditor editor = new MultiChannelListEditor();
					model.setEditor(editor);
					editor.addObserver(model.getView());
					editor.show();
					
				}
			});
		}
		return openMenuItem;
	}

	public void update(Observable sender, Object aspect) {
		if (aspect.equals("closed")) {
			model.handleListEditorClosed();
		}
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
