package gui;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JFrame;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.BoxLayout;
import java.awt.Toolkit;

public class ProgressBarView extends JFrame implements Observer {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private JPanel jContentPane = null;
	private JProgressBar jProgressBar = null;
	private JTextField jTextField = null;
	private JPanel jPanel = null;

	private ProgressBar model;

	public void update(Observable sender, Object aspect) {
		if (aspect.equals("min")) this.handleMinChanged();
		if (aspect.equals("max")) this.handleMaxChanged();
		if (aspect.equals("progress")) this.handleProgressChanged();
		if (aspect.equals("text")) this.handleTextChanged();
	}

	private void handleTextChanged() {
		this.getJTextField().setText(model.getText());
	}

	private void handleProgressChanged() {
		this.getJProgressBar().setValue(model.getProgress());
		if (model.getProgress()>=model.getMax()) {
			Toolkit.getDefaultToolkit().beep();   
			if (model.isCloseWhenFinished()) {
				this.dispose();
			}
		}
	}

	private void handleMaxChanged() {
		this.getJProgressBar().setMaximum(model.getMax());
	}

	private void handleMinChanged() {
		this.getJProgressBar().setMinimum(model.getMin());
	}

	/**
	 * This is the default constructor
	 */
	public ProgressBarView() {
		super();
		initialize();
	}

	/**
	 * This is the default constructor
	 */
	public ProgressBarView(ProgressBar model) {
		super();
		this.model = model;
		model.addObserver(this);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 76);
		this.setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("/icon.gif")));
		this.setContentPane(getJContentPane());
		this.setTitle("progress");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel(), java.awt.BorderLayout.NORTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jProgressBar	
	 * 	
	 * @return javax.swing.JProgressBar	
	 */
	private JProgressBar getJProgressBar() {
		if (jProgressBar == null) {
			jProgressBar = new JProgressBar();
			jProgressBar.setStringPainted(true);
			jProgressBar.setMinimum(model.getMin());
			jProgressBar.setMaximum(model.getMax());
			jProgressBar.setValue(model.getProgress());
		}
		return jProgressBar;
	}

	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new JTextField();
			jTextField.setText(model.getText());
			jTextField.setEnabled(false);
			jTextField.setEditable(false);
		}
		return jTextField;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BoxLayout(getJPanel(), BoxLayout.Y_AXIS));
			jPanel.add(getJTextField(), null);
			jPanel.add(getJProgressBar(), null);
		}
		return jPanel;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
