// Volker Bcker, 16.01.2006: changed all occurences of private to protected

// QT4D Stack Writer Faster - An ImageJ Plug-in to Generate Quicktime Movies
// By Jon Hardin
// June 14th, 2005

package movie;
import ij.plugin.frame.*;
import ij.*;
import ij.process.*;
import ij.io.*;
import ij.gui.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.Vector;
import java.lang.Number;

import javax.swing.*;
import javax.swing.event.*;

import quicktime.qd.*;
import quicktime.*;
import quicktime.std.*;
import quicktime.io.*;
import quicktime.std.image.*;
import quicktime.std.movies.*;
import quicktime.std.movies.media.*;
import quicktime.util.*;
import quicktime.app.view.*;
import quicktime.app.QTFactory;
import quicktime.std.qtcomponents.*;

/*
	This plugin uses QuickTime for Java to save the active stack as a QuickTime movie.
	It is based on the QT for Java  CreateMovie demo.
	
	It has been modified to use a single file TIFF buffer by Jon Hardin.
	It also does not bother to synchronize the file writing routines.
        
    Some of the basic code was adapted from the MovieWriter plugin written by Wayne Rasband, NIH.

    The CompressionSettings class was modified from code by Charles Thomas, LOCI project,
    Univ of Wisconsin-Madison.
        
    Progress bar code was adapted from an online course hosted by Dave Marshall at
    http://www.cs.cf.ac.uk/Dave/HCI/HCI_Handout_CALLER/node125.html
        
    The basics of the image compression was adapted from Brad Lowe brad@nuspectra.com
    based on various QT sample code.

	Kludgy workaround for bugs in the ImageCompressionDialog.requestSequenceSettings() call in 
	QTJ 6.1/Java 1.4.1 added.
 */
 
public class QT_Stack_Writer_fast extends PlugInFrame implements StdQTConstants, Errors
{
	protected int numFrames;
	protected int kWidth;
	protected int kHeight;
	protected File soundFile;
	protected String title;
	protected ImageStack stack;
    public int myIndex;
	public boolean cancelOperation = false;
    public boolean workaround = false;

	// Compressor Settings:
	static int comp_type = kVideoCodecType;
	static int spatial_quality = codecNormalQuality;
	static float frame_rate = 8;
	static int key_frame_rate = 10;
    public int depth;
        
    // Crunch Stuff:
    QTFile movieDest;
    int codecQuality = codecNormalQuality;
    CodecName codecName = null;	
    int maxFrames = -1;
    int curFrame = 0;
    QTFile images[];
    boolean flatten = false;
    static String defaultCodecName = "Animation";

    int kVidTimeScale = 600;		// 1 second = 600
    int defaultFrameDuration = 60;	// frames / 600 = 1/10 sec
    static boolean qtOpen = false;
    static String shortVersion = "QT4D Writer by Jeff and Jon Hardin";
    
    // List of image types to allow for importing. More can be added; keep lowercase:
    final static String okTypes[] = {".jpg", ".jpeg", ".gif", ".tif", ".tiff", ".bmp", ".png", ".pct"};
    
    int numPlanes = -1;
    double timeInterval = -1;
    double spaceInterval = -1;
    String movieInfo = null;
    
    // Other crunching variables:											// CHANGE MADE!
	boolean useNumbersAsSuffix = true;
	String rootName = null;
	String suffixName = null;
	String dir = null;
    String foundName = null;
    String dstFile = "out.mov";
    String codec = defaultCodecName;
    int frameDuration = 0;
    Vector myList = new Vector();
    int quality = -1;
    boolean all = false;
    String dir1 = null;
    
	public QT_Stack_Writer_fast()
	{
		super("Temporary");
	}		

	public void run(String arg)
	{
	    String mySystemString = System.getProperty("java.version");
		double myQTJVersion = (double) QTBuild.getVersion() +  (double) QTBuild.getSubVersion()/10;
		//following updated to account for JVM 1.5
			if (((mySystemString.indexOf("1.4") >= 0) || (mySystemString.indexOf("1.5") >= 0)) && (myQTJVersion >= 6.1))
				{
					// Systems with QTJ 6.1 & Java VM 1.4 or higher require a workaround for now, so set boolean to true.
					workaround = true;
				}

       	saveAsMovie();
	}

	public void saveAsMovie()
	{
        try
		{
			QTSession.open();
    		Crunch converter = new Crunch();
			
			// We need to see if we have a valid, multi-slice stack:
            ImagePlus imp = WindowManager.getCurrentImage();
			if (imp == null)
			{
				// A Type #1 Error occurs when there is no open ImagePlus object.
				IJ.showMessage("This plug-in requires a stack! (Error #1)");
				return;
			}
			else if (imp.getStackSize() <= 1)
			{
				// A Type #2 Error occurs when the open ImagePlus object only contains a single slice.
				IJ.showMessage("This plug-in requires a stack! (Error #2)");
				return;
			}
			
			// IJ.showMessage("Stack Size: " + imp.getStackSize());
			ImageStack stack = imp.getStack();
			
			// See if we have a list of image files to convert into a movie:
			if (stack != null)
			{
		    	int sz = stack.getSize();
		    	
				// Get the current slice and activate the next one:
		    	int currentSlice = imp.getCurrentSlice();
				imp.setSlice(curFrame+1);
		
				// Get current image processor from stack.
	            ImageProcessor ip = imp.getProcessor();
	            ImageProcessor newip = ip.createProcessor(ip.getWidth(), ip.getHeight());
	            newip.setPixels(ip.getPixelsCopy());
	            
	            // Create new image corresponding to this slice.
            	ImagePlus im_temp = new ImagePlus("Foo", newip);
            	
        		// Write it out to a temporary image file:
            	FileSaver fs = new FileSaver(im_temp);
            	Object waitSync = new Object();
            	synchronized(waitSync)
            	{
	            	while (fs.saveAsTiff("thumbnail_123456.tif") == false)
	            	{
	            		waitSync.wait();
	            	}
            	}
		
	            // Now we have to import the temp file into Quicktime for Java:
	            QTFile imageFile2 = new QTFile(new File("thumbnail_123456.tif"));
                
				// Get the compression settings from the user:
        		getCodecSettings(imageFile2);
				if (cancelOperation == true)
				{
					return;
				}
                
				quality = spatial_quality;
				if (quality >= 0)
				{
					converter.setQuality(quality);
				}
                
                converter.setCodec(comp_type);
                frameDuration = (int) ((1/frame_rate)*600);
                if (frameDuration > 0) converter.setFrameDuration(frameDuration);
                
				// Ask the user where the movie should be saved on the hard drive:
                title = imp.getTitle();
                SaveDialog sd = new SaveDialog("Save QuickTime Movie as...", title, ".mov");
				dir = sd.getDirectory();
				String name = sd.getFileName();
				if (name == null)
				{
					throw new QTIOException (userCanceledErr, "");
				}
				dstFile = dir + name;
		
				// If nothing has gone wrong, use our Crunch class (converter) to export the movie:
				if (!all)
				{
					converter.convert(new QTFile(dstFile));
				}
			}
			else
			{
				// A Type #2 Error occurs when the open ImagePlus object only contains a single slice.
				IJ.showMessage("This plug-in requires a stack! (Error #2)");
				return;
			}

			QTSession.close();
			dispose();                
		}
		catch (NoClassDefFoundError e)
		{
			IJ.error("QuickTime for Java required");
		}
		catch (Exception e)
		{
			QTSession.close();
		}
    }
    
    void printStackTrace(Exception e)
	{
		CharArrayWriter caw = new CharArrayWriter();
		PrintWriter pw = new PrintWriter(caw);
		e.printStackTrace(pw);
		IJ.write(caw.toString());
	}
    
 	void getCodecSettings(QTFile test_file) throws Exception
    {
		ImageCompressionDialog comp_dlog = null;
		SpatialSettings spatial_settings = null;
		TemporalSettings temporal_settings = null;
		DataRateSettings data_rate_settings = null;
		QDRect test_rect = null;
		int color_depth = 0;
		CodecComponent comp = CodecComponent.anyCodec;
		int temporal_quality = codecNormalQuality;
		int data_rate = 1000;
		int frame_duration = 8;
		boolean cancelled = false;
		int test_flags = 0;
		int flags = 0;
        PixMap test_pix_map = null;

		// Get the standard QT compression dialog:
		comp_dlog = new ImageCompressionDialog();
		
		// Set the default image:
		GraphicsImporter myImporter = new GraphicsImporter(test_file);
		comp_dlog.setTestImagePict(myImporter.getAsPicture(), test_rect, test_flags);

		// Set up the default settings:
		spatial_settings = new SpatialSettings(comp_type, comp, color_depth, spatial_quality);
		comp_dlog.setInfoSpatialSettings(spatial_settings);
		temporal_settings = new TemporalSettings(temporal_quality, frame_rate, key_frame_rate);
		comp_dlog.setInfoTemporalSettings(temporal_settings);
		data_rate_settings = new DataRateSettings(data_rate, frame_duration, codecMinQuality, codecMinQuality);
		comp_dlog.setInfoDataRateSettings(data_rate_settings);
		
		// Set the default such that BEST DEPTH is not an option:
		flags &= ~scShowBestDepth;
		comp_dlog.setInfoPreferences(flags);

		cancelOperation = true;

		// Display the dialog for the user:
		try
		{
			// Use a thread, which protects the dialog from crashing due to synchronization problems:
			final ImageCompressionDialog cdl = comp_dlog;
			Thread t = new Thread()
			{
				public void run()
				{
					try
					{
						if (workaround)
						{
							// JVM 1.4:
							cdl.requestImageSettings();
						}
						else
						{
							// JVM 1.3 or lower:
							cdl.requestSequenceSettings();
						}
					}
					catch (QTException qte)
					{
						qte.printStackTrace();
					}
				}
			};
			t.start();
			while (t.isAlive())
			{
				// Yield 100% time to the thread to synchronize it:
				Thread.yield();
			}
			comp_dlog = cdl;
		}
		catch (Error er)
		{
			throw new Error("User cancelled");
		}
		
		if (!cancelled)
		{
			// Get the user settings:
			spatial_settings = comp_dlog.getInfoSpatialSettings();
            depth = spatial_settings.getDepth();
			comp_type = spatial_settings.getCodecType();
			spatial_quality = spatial_settings.getSpatialQuality();	
            
            temporal_settings = comp_dlog.getInfoTemporalSettings();
			frame_rate = temporal_settings.getFrameRate();
            key_frame_rate = temporal_settings.getKeyFrameRate();
            cancelOperation = false;
			 
 			// If we are using a particular computer setup (JVM 1.4), we need to use a workaround to get the frame rate:
 			if ((!cancelled) && (workaround))
 			{
                GenericDialog gd = new GenericDialog("Frame Rate Options", IJ.getInstance());
                double xinitial = 8.0;
                double yinitial = 0.0;
                gd.addNumericField("Frame rate (fps):",xinitial, 1);
                gd.addNumericField("Key frames:", yinitial,0);
				gd.showDialog();
				if (gd.wasCanceled())
				{
					cancelOperation = true;
				}
				else
				{		
                    frame_rate = (float) gd.getNextNumber();
                    key_frame_rate = (int) gd.getNextNumber();
					cancelOperation = false;
				}
				
				//IJ.showMessage("Final Frame Rate: " + frame_rate);
				//IJ.showMessage("Keyframe Rate: " + key_frame_rate);
			}
		}

		// Delete the temp file:
		test_file.delete();
	}
	
	public class GWorldToPict extends Object implements QDDrawer
	{
		/*
		This class is used to convert from an ImagePlus to a Pict image, all in RAM.
		Unfortunately, there is a major bottleneck associated with creating QDColor objects.
		*/
		
		ImagePlus sourceIM;
		
		public GWorldToPict()
	    {
	    	// Hey!
	    }
	    
	    public Pict doStuff(ImageProcessor newip) throws QTException
	    {
	    	try
	        {
	            // QTSessionCheck.check();
	            sourceIM = new ImagePlus("Foo", newip);
	            QDRect bounds = new QDRect (0, 0, sourceIM.getWidth(), sourceIM.getHeight());
	            ImageDescription imgDesc = new ImageDescription(QDConstants.k32RGBAPixelFormat);
	            imgDesc.setHeight (bounds.getHeight());
	            imgDesc.setWidth (bounds.getWidth());
	            QDGraphics gw = new QDGraphics (imgDesc, 0);
	            
	            OpenCPicParams params = new OpenCPicParams(bounds);
	
	            Pict pict = Pict.open (gw, params);
	            gw.beginDraw(this);
	
	            pict.close();
	            return pict;
	        }
	        catch (QTException qte)
	        {
	            qte.printStackTrace();
	        }
	        
	        return null;
	    }
	
	    public void draw (QDGraphics gw) throws QTException
	    {
	        QDRect bounds = gw.getBounds();
	        gw.setBackColor (QDColor.white);
	        gw.setForeColor(QDColor.green);
	        gw.eraseRect (bounds);
	        gw.penSize(1,1);
	        
	        // IJ.showStatus("Hey! 1");
	        // IJ.showMessage("Width: " + sourceIM.getWidth());
	        // IJ.showMessage("Height: " + sourceIM.getHeight());
	        
	        for (int i = 0; i < sourceIM.getWidth(); ++i)
	        {
	        	for (int j = 0; j < sourceIM.getHeight(); ++j)
	        	{
	        		// IJ.showStatus("I = " + i + "  J = " + j);
	        		gw.moveTo(i,j);
	        		int k[] = sourceIM.getPixel(i,j);
	        		gw.setForeColor(new QDColor(k[1],k[2],k[3]));	// BOTTLENECK!!!
	        		gw.line(i,j);
	        		
	        		/*
	        		Returns the pixel value at (x,y) as a 4 element array.
	        		Grayscale values are retuned in the first element.
	        		RGB values are returned in the first 3 elements.
	        		For indexed color images, the RGB values are returned
	        		in the first 3 three elements and the index (0-255) is returned in the last.
	        		*/
	        	}
	        }
	        
	        IJ.showStatus("Done!");
	    }
	
	}

	public class Crunch
	{
		QTFile images[];
		boolean stopProgram;
		JFrame myFrame;
		JLabel myLabel;
		JProgressBar progress;
		
		public Crunch() throws QTException
		{
			// No initialization is needed here...
		}
		
		public void setCodec(int myCType) throws Exception
		{
			// Set the desired codec for exporting the movie with QT4J:
			CodecNameList list = new CodecNameList(1);
			for (int i=0;i<list.getCount();i++)
			{
				CodecName item = list.getNth(i);
				if (myCType == item.getCType())
				{
					codecName=item;
					return;
				}
			}
		}

		// Set the spatial and temporal quality:
		public void setQuality(int i)
		{
			codecQuality=i;
		}
		
		// Set the individual frame duration:
		public void setFrameDuration(int i)
		{
			defaultFrameDuration = i;
		}
		
		// Get the number of slices in the stack:
		public int getMovieFrameCount() throws Exception
		{
			return images.length;
		}
		
		// Overloaded method to take File objects:
		public void convert(File movieDest) throws Exception
		{
			convert(new QTFile(movieDest));
		}

		// Overloaded method to take QTFile objects:
		public void convert(QTFile movieDest) throws Exception        
		{
			if (codecName == null)
			{
				setCodec(kVideoCodecType);
			}
			
			stopProgram = false;
			Movie theMovie = Movie.createMovieFile (movieDest, kMoviePlayer, createMovieFileDeleteCurFile | createMovieFileDontCreateResFile);
		
			// Make a window to display the progress and cancel buttons:
		    progress = new JProgressBar();
			JButton button = new JButton();
		    JPanel topPanel = new JPanel();
			myFrame = new JFrame("Compressing images...");
		    
		    myFrame.setSize(310, 130);
			topPanel = new JPanel();
			topPanel.setPreferredSize(new Dimension(310, 130));
			myFrame.getContentPane().add(topPanel);
		
			// Create a label and progress bar:
			myLabel = new JLabel("");
			myLabel.setPreferredSize(new Dimension(280, 24));
			myLabel.setForeground(Color.black);
			topPanel.add(myLabel);
		
			progress = new JProgressBar();
			progress.setPreferredSize(new Dimension(300, 20));
			progress.setMinimum(0);
			progress.setMaximum(90);
			progress.setValue(0);
			progress.setBounds(20, 35, 260, 20);
			progress.setForeground(Color.blue);
			topPanel.add(progress);
		
			button = new JButton("Stop Compression");
			topPanel.add(button);
			button.addActionListener(new ActionListener()
			{
		    	public void actionPerformed(ActionEvent e)
		    	{
					stopProgram = true;
					myFrame.dispose();
				}
			});
		    myFrame.show();
		
			// Create a blank movie object and add a video track to it, then export it to a file:
			try
			{	
				addVideoTrack(theMovie);
				
				OpenMovieFile outStream = OpenMovieFile.asWrite (movieDest); 
				theMovie.addResource(outStream, movieInDataForkResID, movieDest.getName());
				outStream.close();
		
				if (flatten)
				{
					theMovie.flatten(0, movieDest, kMoviePlayer, IOConstants.smSystemScript, createMovieFileDeleteCurFile | createMovieFileDontCreateResFile,
					movieInDataForkResID, movieDest.getName());
				}
				
				myFrame.dispose();
				IJ.showStatus ("Finished movie. Duration:" + theMovie.getDuration() + " Length:" + movieDest.length());
				
				theMovie.disposeQTObject();
				theMovie = null;
			}
			catch (Exception qte)
			{
				qte.printStackTrace(); 
		
				if (movieDest.exists())
				{
					movieDest.delete();
				}
				throw qte;
			}
		}
		
		int getFrameDuration()
		{
			return defaultFrameDuration;	
		}
		
		// This removes a directory and all of its contents from the hard drive:
		public boolean deleteDir(File dir)
	    {
	        if (dir.isDirectory())
	        {
	            String[] children = dir.list();
	            for (int i=0; i < children.length; i++)
	            {
	                boolean success = deleteDir(new File(dir, children[i]));
	                if (!success)
	                {
	                    return false;
	                }
	            }
	        }
	    
	        // The directory is now empty, so delete it:
	        return dir.delete();
	    }

		protected void addVideoTrack (Movie theMovie) throws QTException, Exception 
		{
			// Extract the stack, and calculate its attributes:
			ImagePlus imp = WindowManager.getCurrentImage();
			ImageStack stack = imp.getStack();
		    int sz = stack.getSize();
			int numFrames = sz;
			if (maxFrames > 0 && numFrames > maxFrames)	
			{
				numFrames = maxFrames;
			}
			
			// Configure the progress bar and other QT4J objects:
			progress.setMaximum(numFrames);
			QDRect rect = null;
			QDGraphics gw = null;
			Track vidTrack = null;
			VideoMedia vidMedia = null;
			QTHandle imageHandle = null;
			RawEncodedImage compressedImage = null;
			CSequence seq = null;
			ImageDescription desc = null;
			
			// Create a directory for the temporary files to live in:
        	String dir1 = "123456_temp/";
			File tempDir = new File(dir1);
			boolean success = tempDir.mkdir();
		
			// Loop through all of the slices in the stack:
			for (curFrame = 0; curFrame < numFrames; curFrame++) 
			{
				try
				{
					// Grab the current slice, and activate the next one:
					int currentSlice = imp.getCurrentSlice();
					imp.setSlice(curFrame+1);
			
					// Get current image processor from stack.
		            ImageProcessor ip = imp.getProcessor();
		            ImageProcessor newip = ip.createProcessor(ip.getWidth(), ip.getHeight());
		            newip.setPixels(ip.getPixelsCopy());
		            
		            // Create new image corresponding to this slice.
	            	ImagePlus im = new ImagePlus("Foo", newip);
	            	
	            	// Write it out to a temporary image file:
	            	FileSaver fs = new FileSaver(im);
	            	
					// This is a slower version of the exporter, but it is synchronized:
	            	/*
	            	Object waitSync = new Object();
	            	synchronized(waitSync)
	            	{
		            	while (fs.saveAsTiff(dir1 + "123456_temp0" + curFrame + ".tif") == false)
		            	{
		            		waitSync.wait();
		            	}
	            	}
	            	*/
		
					// This version is faster, and doesn't need to be synchronized, so we will use it instead:
					if (fs.saveAsTiff(dir1 + "123456_temp0" + curFrame + ".tif") == false)
	            	{
	            		// There was a problem writing out the temporary TIFF file:
						IJ.showStatus("Error writing temporary file!");
	            	}
	            }
	            catch (Exception qte)
	            {
	            	IJ.showMessage("Darn it! There was an error...");
	            	qte.printStackTrace();
	            	throw qte;
	            }
	            
	            // Now we have to import the temp file into Quicktime for Java:
	            QTFile imageFile2 = new QTFile(new File(tempDir, "123456_temp0" + curFrame + ".tif"));
				
				// Import the image into QuickTime:
				GraphicsImporter myGraphicsImporter = new GraphicsImporter (imageFile2);
				GraphicsImporterDrawer drawer = new GraphicsImporterDrawer (myGraphicsImporter);
				
				if (curFrame == 0)
				{
					// Do initial setup, now that we can find out the width and height of first image:		
					rect = myGraphicsImporter.getBoundsRect();
					gw = new QDGraphics (rect);
					vidTrack = theMovie.addTrack (rect.getWidth(), rect.getHeight(), 0);
					vidMedia = new VideoMedia (vidTrack, kVidTimeScale); 
					vidMedia.beginEdits();
					
					int size = QTImage.getMaxCompressionSize (gw, rect, depth, codecQuality, codecName.getCType(), codecName.getCodecComponent());
					imageHandle = new QTHandle (size, true);
					imageHandle.lock();
					compressedImage = RawEncodedImage.fromQTHandle(imageHandle);
					seq = new CSequence (gw, rect, depth, codecName.getCType(), codecName.getCodecComponent(), codecQuality, codecQuality, QT_Stack_Writer_fast.key_frame_rate, null, 0);
					
					desc = seq.getDescription();
				}
		
				myLabel.setText("Compressing frame " +  (curFrame+1) + " of " + numFrames);
				progress.setValue(curFrame + 1);
				
				// If the user presses cancel on our progress bar window, stop everything:
				if (stopProgram)
				{
					// Since we just break the loop, we should still have a working movie that can be
					// flattened and then exported just like it would be normally:
					break;
				}
		
				// Redrat the progress window:
				drawer.setGWorld(gw);
				drawer.redraw(null);
				
				// Compress the imported frame for inclusion in the Quicktime movie:
				CompressedFrameInfo info = seq.compressFrame(gw, rect, codecFlagUpdatePrevious, compressedImage);
				boolean isKeyFrame = info.getSimilarity() == 0;
				
				vidMedia.addSample (imageHandle, 
									0,										// dataOffset,
									info.getDataSize(),
									getFrameDuration(),						// desired time per frame	
									desc,
									1,										// one sample
									(isKeyFrame ? 0 : mediaSampleNotSync));	// no flags
				
				myGraphicsImporter.disposeQTObject();
				
				// Delete the temporary file for the current iteration:
				File deltemp = new File(tempDir, "123456_temp0" + curFrame + ".tif");
				deltemp.delete();
			}
		
			// Finalize the edited video track:
			vidMedia.endEdits();
			
			// Insert the edited video track into the movie:
			int kTrackStart	= 0;
			int kMediaTime = 0;
			int kMediaRate	= 1;
			vidTrack.insertMedia (kTrackStart, kMediaTime, vidMedia.getDuration(), kMediaRate);

			// Remove our temporary directory:
			deleteDir(tempDir);
		}
	}
}


