package processingOperations;

import java.awt.geom.Point2D;

import analysisOperations.ComputeDifferenceOperation;

import operations.Operation;
import utils.ImageTransformer;
import gui.Option;
import gui.Registration;
import ij.ImagePlus;

public class AlignImagesOperation extends Operation {
	protected Option initialStepWidth;
	/**
	 * 
	 */
	private static final long serialVersionUID = 3100855032584906195L;
	
	protected ImagePlus secondInputImage;
	protected Registration registration;
	protected ComputeDifferenceOperation differenceCalculator;
	
	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		parameterTypes = new Class[2];
		parameterTypes[0] = Class.forName("ij.ImagePlus");
		parameterTypes[1] = Class.forName("ij.ImagePlus");
		parameterNames = new String[2];
		parameterNames[0] = "InputImage";
		parameterNames[1] = "SecondInputImage";
		resultTypes = new Class[2];
		resultTypes[0] = Class.forName("ij.ImagePlus");
		resultTypes[1] = Class.forName("gui.Registration");
		resultNames = new String[2];
		resultNames[0] = "Result";
		resultNames[1] = "Registration";
		optionsNames = new String[1];
		optionsNames[0] = "initial step width";
	}

	public Registration getRegistration() {
		return registration;
	}

	public void setRegistration(Registration registration) {
		this.registration = registration;
	}

	public ImagePlus getSecondInputImage() {
		return secondInputImage;
	}

	public void setSecondInputImage(ImagePlus secondInputImage) {
		this.secondInputImage = secondInputImage;
	}

	protected void cleanUpInput() {
		super.cleanUpInput();
		this.setSecondInputImage(null);
	}
	
	public void doIt() {
		Point2D startPointFirstImage = this.findStartOfImage(this.getInputImage());
		Point2D startPointSecondImage = this.findStartOfImage(this.getSecondInputImage());
		int width = this.getImageWidth(this.getSecondInputImage(), startPointSecondImage);
		int height = this.getImageHeight(this.getSecondInputImage(), startPointSecondImage);
		ImagePlus secondInputImage = this.getSecondInputImage();
		result = this.getCopyOfOrReferenceTo(secondInputImage, secondInputImage.getTitle());
		int xDiff = (int)(startPointFirstImage.getX() - startPointSecondImage.getX());
		int yDiff = (int)(startPointFirstImage.getY() - startPointSecondImage.getY());
		this.setResult(ImageTransformer.shift(result, xDiff, yDiff));
		this.setResult(ImageTransformer.shift(result, (int)(-width/2.0), (int)(-height/2.0)));
		this.align(this.getInitialStepWidth());
		this.align(1);
	}

	protected void align(int step) {
		double currentQuality = this.getCurrentQuality();
		System.out.println("quality: " + currentQuality);
		
	}

	protected double getCurrentQuality() {
		ComputeDifferenceOperation calc = this.getDifferenceCalculator();
		calc.setShowResult(false);
		calc.setFirstImage(this.getInputImage());
		calc.setSecondImage(this.getResult());
		calc.run();
		double result = calc.getDifference().getValue("average difference",0);
		return result;
	}

	protected ComputeDifferenceOperation getDifferenceCalculator() {
		if (this.differenceCalculator==null) this.differenceCalculator = new ComputeDifferenceOperation();
		return this.differenceCalculator;
	}

	private int getImageHeight(ImagePlus secondInputImage2, Point2D start) {
		int x = (int)start.getX();
		int y = (int) start.getY();
		for (; y<inputImage.getHeight(); y++) {
				if (inputImage.getPixel(x,y)[0] == 0) {
					break; 
				}
		}
		return y-(int)start.getY();
	}
		
	private int getImageWidth(ImagePlus secondInputImage2, Point2D start) {
		int x = (int)start.getX();
		int y = (int) start.getY();
		for (; x<inputImage.getWidth(); x++) {
				if (inputImage.getPixel(x,y)[0] == 0) {
					break; 
				}
		}
		return x-(int)start.getX();
	}

	protected Point2D findStartOfImage(ImagePlus inputImage) {
		int x=0;
		int y=0;
		boolean finished = false;
		for (x=0; x<inputImage.getWidth(); x++) {
			for (y=0; y<inputImage.getHeight(); y++) {
				if (inputImage.getPixel(x,y)[0] > 0) {
					finished = true;
					break; 
				}
			}
			if (finished) break;
		}
		return new Point2D.Float(x,y);
	}
	
	public void connectOptions() {
		this.initialStepWidth= (Option) this.options.getOptions().elementAt(0);
	}

	public int getInitialStepWidth() {
		return initialStepWidth.getIntegerValue();
	}

	public void setInitialStepWidth(int initialStepWidth) {
		this.initialStepWidth.setValue(Integer.toString(initialStepWidth));
	}
	
	protected void setupOptions() {
		super.setupOptions();
		this.setInitialStepWidth(10);
	}
}
