package processingOperations;

import gui.BooleanOption;
import gui.Option;
import gui.Options;
import ij.IJ;
import operations.FilterOperation;

public class EnhanceSpotsOperation extends FilterOperation {
	/* Calls the plugin LoG 3D
	 * Spot detector based on 3D Laplacian of Gaussian or Mexican Hat
	 * Daniel Sage
	 * http://bigwww.epfl.ch/sage/soft/LoG3D/index.html
	 */ 
	
	Option sigmaX;
	Option sigmaY;
	Option sigmaZ;
	Option volume;
	/**
	 * 
	 */
	private static final long serialVersionUID = 4238604458936674065L;

	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		optionsNames = new String[4];
		optionsNames[0] = "sigma x";
		optionsNames[1] = "sigma y";
		optionsNames[2] = "sigma z";
		optionsNames[3] = "volume";
	}
	
	public void runFilter() {
		int volume = 0;
		if (this.getVolume()) volume = 1;
		String parameter = "sigmax=" + this.getSigmaX() + 
						  " sigmay=" + this.getSigmaY() + 
						  " sigmaz=" + this.getSigmaZ() +
						  " displaykernel=0" +
						  " volume=" + volume;
		IJ.run("LoG 3D", parameter);
	}
	
	protected void setupOptions() {
		options = new Options(); 
		options.setName(this.name() + " options");
		String[] optionsNames = this.getOptionsNames();
		this.sigmaX = new Option();
		this.setSigmaX(3);
		this.sigmaX.setName(optionsNames[0]);
		this.sigmaX.setShortHelpText("The filter size in x-direction.");
		options.add(this.sigmaX);
		this.sigmaY = new Option();
		this.setSigmaY(3);
		this.sigmaY.setName(optionsNames[1]);
		this.sigmaY.setShortHelpText("The filter size in y-direction.");
		options.add(this.sigmaY);
		this.sigmaZ = new Option();
		this.setSigmaZ(3);
		this.sigmaZ.setName(optionsNames[2]);
		this.sigmaZ.setShortHelpText("The filter size in z-direction.");
		options.add(this.sigmaZ);
		this.volume = new BooleanOption();
		this.setVolume(false);
		this.volume.setName(optionsNames[3]);
		this.sigmaX.setShortHelpText("Select volume processing or slice-by-slice processing.");
		options.add(this.volume);
	}

	public void connectOptions() {
		this.sigmaX = (Option) this.options.getOptions().elementAt(0);
		this.sigmaY = (Option) this.options.getOptions().elementAt(1);
		this.sigmaZ = (Option) this.options.getOptions().elementAt(2);
		this.volume = (BooleanOption) this.options.getOptions().elementAt(3);
	}
	
	public int getSigmaX() {
		return sigmaX.getIntegerValue();
	}

	public void setSigmaX(int sigmaX) {
		this.sigmaX.setValue(Integer.toString(sigmaX));
	}

	public int getSigmaY() {
		return sigmaY.getIntegerValue();
	}

	public void setSigmaY(int sigmaY) {
		this.sigmaY.setValue(Integer.toString(sigmaY));
	}
	
	public int getSigmaZ() {
		return sigmaZ.getIntegerValue();
	}

	public void setSigmaZ(int sigmaZ) {
		this.sigmaZ.setValue(Integer.toString(sigmaZ));
	}
	
	public boolean getVolume() {
		return volume.getBooleanValue();
	}

	public void setVolume(boolean value) {
		this.volume.setValue(Boolean.toString(value));
	}
}
